'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoadEnvironmentVariables = void 0;
const fs_1 = __importDefault(require("@supercharge/fs"));
const dotenv_1 = __importDefault(require("dotenv"));
class LoadEnvironmentVariables {
    /**
     * Create a new instance.
     *
     * @param app Application
     */
    constructor(app) {
        this.app = app;
    }
    /**
     * Boot application services.
     */
    async bootstrap() {
        await this.loadEnvironment();
    }
    /**
     * Load the contents of the configured environment file. Throws an error
     * in case the environment file does not exist, an error will be thrown.
     *
     * @throws
     */
    async loadEnvironment() {
        await this.loadDefaultEnvironmentFile();
        await this.loadSpecificEnvironmentFile();
    }
    /**
     * Load the app’s configured environment file.
     */
    async loadDefaultEnvironmentFile() {
        const envPath = this.app.environmentFilePath();
        if (await fs_1.default.notExists(envPath)) {
            throw new Error(`Invalid environment file. Cannot find env file "${envPath}".`);
        }
        await this.loadEnvironmentFile(this.app.environmentFilePath(), { override: false });
    }
    /**
     * Load the given environment `file` content into `process.env`.
     */
    async loadEnvironmentFile(path, options) {
        dotenv_1.default.config({ path, ...options });
    }
    /**
     * Check if an environment file exists for the configured `NODE_ENV`. For example,
     * if `NODE_ENV=testing`, check whether a `.env.testing` file exists and loadd it.
     */
    async loadSpecificEnvironmentFile() {
        const env = this.app.env().get('NODE_ENV').toLowerCase();
        if (!env) {
            return;
        }
        const envFilePath = this.app.resolveFromBasePath(this.app.environmentPath(), `.env.${env}`);
        if (await fs_1.default.exists(envFilePath)) {
            await this.loadEnvironmentFile(envFilePath, { override: true });
        }
    }
}
exports.LoadEnvironmentVariables = LoadEnvironmentVariables;
