'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsoleKernel = void 0;
const fs_1 = __importDefault(require("@supercharge/fs"));
const collections_1 = __importDefault(require("@supercharge/collections"));
const goodies_1 = require("@supercharge/goodies");
const console_1 = require("@supercharge/console");
const bootstrappers_1 = require("../bootstrappers");
class ConsoleKernel {
    /**
     * Create a new console kernel instance.
     *
     * @param {Application} app
     */
    constructor(app) {
        this.meta = { app };
    }
    /**
     * Creates a new console kernel instance for the given `app`.
     *
     * @param {Application} app
     */
    static for(app) {
        return new this(app);
    }
    /**
     * Returns the application instance.
     *
     * @returns {Application}
     */
    app() {
        return this.meta.app;
    }
    /**
     * Returns the list of application bootstrappers.
     */
    bootstrappers() {
        return [
            bootstrappers_1.HandleExceptions,
            bootstrappers_1.LoadEnvironmentVariables,
            bootstrappers_1.LoadConfiguration,
            bootstrappers_1.RegisterServiceProviders,
            bootstrappers_1.BootServiceProviders
        ];
    }
    /**
     * Handle an incoming console command for the given `input`.
     *
     * @param {Array} input - command line arguments (process.argv)
     *
     * @returns {Promise}
     */
    async run(input) {
        await this.bootstrap();
        await this.craft().run(input);
    }
    /**
     * Prepare the console application by running the configured bootstrappers.
     * This method doesn’t register configured commands in the application.
     * It prepares the console application which is useful for testing.
     */
    async prepare() {
        await this.app().bootstrapWith(this.bootstrappers());
        return this;
    }
    /**
     * Bootstrap the console application for Craft commands.
     */
    async bootstrap() {
        await this.prepare();
        await this.commands();
    }
    /**
     * Register the console commands to the application.
     */
    async commands() {
        //
    }
    /**
     * Register all of the commands in the given `directories`.
     *
     * @param {Array} paths
     */
    async loadFrom(...paths) {
        await (0, collections_1.default)(paths)
            .unique()
            .filter(async (path) => {
            return await fs_1.default.exists(path);
        })
            .flatMap(async (path) => {
            return await fs_1.default.allFiles(path);
        })
            .map(async (commandFile) => {
            return await this.resolve(commandFile);
        })
            .forEach((command) => {
            this.craft().add(command);
        });
    }
    /**
     * Requires the given file from disk.
     *
     * @param {String} commandFile
     *
     * @returns {Command}
     */
    async resolve(commandFile) {
        const CommandCtor = (0, goodies_1.esmRequire)(commandFile);
        return new CommandCtor();
    }
    /**
     * Returns a Craft console application instance.
     *
     * @returns {Craft}
     */
    craft() {
        if (!this.meta.craft) {
            this.meta.craft = new console_1.Application(this.app());
        }
        return this.meta.craft;
    }
}
exports.ConsoleKernel = ConsoleKernel;
