'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpKernel = void 0;
const goodies_1 = require("@supercharge/goodies");
const collections_1 = __importDefault(require("@supercharge/collections"));
const bootstrappers_1 = require("../bootstrappers");
class HttpKernel {
    /**
     * Create a new HTTP kernel instance.
     *
     * @param {Application} app
     */
    constructor(app) {
        this.meta = {
            app,
            bootedCallbacks: [],
            isBootstrapped: false
        };
        this.registerHttpContainerBindings();
        this.register();
    }
    /**
     * Creates a new HTTP kernel instance for the given `app`.
     *
     * @param {Application} app
     */
    static for(app) {
        return new this(app);
    }
    /**
     * Returns the app instance.
     */
    app() {
        return this.meta.app;
    }
    /**
     * Register the HTTP base bindings into the container.
     */
    registerHttpContainerBindings() {
        this.app().singleton('http.kernel', () => this);
    }
    /**
     * Determine whether the HTTP kernel is already boostrapped.
     *
     * @returns {Boolean}
     */
    isBootstrapped() {
        return this.meta.isBootstrapped;
    }
    /**
     * Determine whether the HTTP kernel is not boostrapped yet.
     *
     * @returns {Boolean}
     */
    isNotBootstrapped() {
        return !this.isBootstrapped();
    }
    /**
     * Mark this HTTP kernel as bootstrapped.
     *
     * @returns {this}
     */
    markAsBootstrapped() {
        return (0, goodies_1.tap)(this, () => {
            this.meta.isBootstrapped = true;
        });
    }
    /**
     * Returns the HTTP server instance.
     *
     * @returns {Server}
     */
    server() {
        return this.app().make('server');
    }
    /**
     * Register the booting or booted callbacks. For example, you may use a `booted`
     * callback to send a "ready" signal to PM2 (or any other process manager)
     * informing that your HTTP server is ready to accept incoming requests.
     */
    register() {
        //
    }
    /**
     * Returns the application’s global middleware stack. Every middleware
     * listed here runs on every request to the application.
     *
     * @returns {MiddlewareCtor[]}
     */
    middleware() {
        return [];
    }
    /**
     * Returns the application’s global middleware stack. Every middleware
     * listed here runs on every request to the application.
     *
     * @returns {{ [key: string]: MiddlewareCtor}}
     */
    routeMiddleware() {
        return {};
    }
    /**
     * Register a booted callback that runs after the HTTP server started.
     *
     * @returns {this}
     */
    booted(callback) {
        return (0, goodies_1.tap)(this, () => {
            this.bootedCallbacks().push(callback);
        });
    }
    /**
     * Returns the booted callbacks.
     *
     * @returns {Callback[]}
     */
    bootedCallbacks() {
        return this.meta.bootedCallbacks;
    }
    /**
     * Returns a request handler callback compatible with Node.js’ native HTTP server. This method
     * bootstraps the HTTP server instance by registering routes and middleware before returning
     * it. This request handler callback is useful during testing when sending requests into
     * the HTTP server instance with a community package, like Supertest from visionmedia.
     *
     * @returns {HttpServerHandler}
     */
    async serverCallback() {
        if (this.isNotBootstrapped()) {
            await this.bootstrap();
            await this.runBootedCallbacks();
        }
        return this.server().callback();
    }
    /**
     * Start the HTTP instance listening for incoming requests.
     *
     * @returns {Promise<this>}
     */
    async startServer() {
        await this.bootstrap();
        await this.listen();
        await this.runBootedCallbacks();
        return this;
    }
    /**
     * Start the HTTP server to listen on a local port.
     */
    async listen() {
        await this.server().start();
    }
    /**
     * Stop the HTTP instance.
     *
     * @returns {Promise<this>}
     */
    async stopServer() {
        return await (0, goodies_1.tap)(this, async () => {
            await this.server().stop();
        });
    }
    /**
     * Prepare the application by running the configured bootstrappers. This
     * method doesn’t register routes or middleware to the underlying HTTP
     * server. It prepares the application which is useful for testing.
     */
    async prepare() {
        await this.app().bootstrapWith(this.bootstrappers());
        return this;
    }
    /**
     * Bootstrap the HTTP kernel by loading environment variables and all files
     * within the `config` directory, register and boot all the given service
     * providers, and ultimately register the HTTP middleware and routes.
     */
    async bootstrap() {
        if (this.isBootstrapped()) {
            return;
        }
        await this.prepare();
        this.registerMiddleware();
        this.server().bootstrap();
        this.markAsBootstrapped();
    }
    /**
     * Returns the list of application bootstrappers.
     */
    bootstrappers() {
        return [
            bootstrappers_1.HandleExceptions,
            bootstrappers_1.HandleShutdown,
            bootstrappers_1.LoadEnvironmentVariables,
            bootstrappers_1.LoadConfiguration,
            bootstrappers_1.RegisterServiceProviders,
            bootstrappers_1.BootServiceProviders
        ];
    }
    /**
     * Register the configured middleware stacks to the HTTP server.
     */
    registerMiddleware() {
        this.registerAppMiddleware();
        this.registerRouteLevelMiddleware();
    }
    /**
     * Register the application middleware stack to the HTTP server.
     */
    registerAppMiddleware() {
        this.middleware().forEach((Middleware) => {
            this.server().use(Middleware);
        });
    }
    /**
     * Sync the available middleware to the router.
     */
    registerRouteLevelMiddleware() {
        Object.entries(this.routeMiddleware()).forEach(([name, middleware]) => {
            this.server().useRouteMiddleware(name, middleware);
        });
    }
    /**
     * Run the configured `booted` callbacks.
     */
    async runBootedCallbacks() {
        await this.runCallbacks(this.bootedCallbacks());
    }
    /**
     * Call the given kernal `callbacks`.
     *
     * @param {Callback[]} callbacks
     */
    async runCallbacks(callbacks) {
        await (0, collections_1.default)(callbacks).forEach(async (callback) => {
            await callback();
        });
    }
}
exports.HttpKernel = HttpKernel;
